@@ -12,7 +12,7 @@ sctheme <-
             text = element_text(size = base_size, family = family),
             panel.background = element_rect(fill = "white", colour = NA),
             axis.line = element_line(colour = "black"),
-            axis.ticks = element_line(colour = "black", size = base_size / 20),
+            axis.ticks = element_line(colour = "black", linewidth = base_size / 20),
             axis.title = element_text(face = "bold"),
             axis.text = element_text(size = base_size, family = family),
             axis.text.x = element_text(angle = Xang, hjust = XjusH),
