@@ -175,6 +175,16 @@ updateSubsetCellUI <-
                 renderUI({subsetCell.ui[[paste0("uis", ABcolumn)]]})
         })
     }
+getM <- function(val){
+    val <- max(val, na.rm = TRUE)
+    if (val <= 1)
+        maxv <- round(val, digits = 3)
+    if (val > 1 && val <= 10)
+        maxv <- round(val, digits = 1)
+    if (val > 10)
+        maxv <- ceiling(val)
+    return(maxv)
+}
 updateFilterCellUI <-
     function(
         id,
@@ -229,16 +239,6 @@ updateFilterCellUI <-
                 }
             }
             minv <- floor(min(val, na.rm = TRUE))
-            getM <- function(val){
-                val <- max(val, na.rm = TRUE)
-                if (val <= 1)
-                    maxv <- round(val, digits = 3)
-                if (val > 1 && val <= 10)
-                    maxv <- round(val, digits = 1)
-                if (val > 10)
-                    maxv <- ceiling(val)
-                return(maxv)
-            }
             maxv <- getM(val)
             if(length(val2)>1){
                 minv2 <- floor(min(val2, na.rm = TRUE))
@@ -598,6 +598,9 @@ updateCellInfoPlot <-
         dataSource) {
         cellInfoLabel <- paste0('CellInfo', postfix)
         cellInfoName <- paste0('CellInfoname', postfix)
+        cellInfoXYlimTog <- paste0('CellInfoxylimTog', postfix)
+        cellInfoXlim <- paste0('CellInfoxlim', postfix)
+        cellInfoYlim <- paste0('CellInfoylim', postfix)
         observeEvent(input[[cellInfoLabel]],{
             updateSelectInput(
                 session,
@@ -614,6 +617,55 @@ updateCellInfoPlot <-
             choices = dataSource()$sc1conf$UI,
             selected = dataSource()$sc1def[[paste0("meta", postfix)]]
         )
+        output[[paste0('subsetCellNum', postfix)]] <- 
+            renderText(paste('% of', nrow(dataSource()$sc1meta), 'cells'))
+        observeEvent({getSubsetCellVal(input)},{
+            output[[paste0('subsetCellNum', postfix)]] <- 
+                renderText(paste('% of', getFilteredCellNum(
+                    inpConf=dataSource()$sc1conf,
+                    inpMeta=dataSource()$sc1meta,
+                    dimRedX=input$GeneExprdrX,
+                    dimRedY=input$GeneExprdrY,
+                    cellinfoID=input[[cellInfoLabel]],
+                    cellinfoName=input[[cellInfoName]],
+                    subsetCellKey=input$subsetCell,
+                    subsetCellVal=getSubsetCellVal(input),
+                    subsetCellPct=100
+                ), 'cells'))
+        })
+        observeEvent(input[[cellInfoXYlimTog]],{
+            val <- dataSource()$sc1meta[[
+                dataSource()$sc1conf[
+                    dataSource()$sc1conf$UI == input$GeneExprdrX]$ID]]
+            minv <- floor(min(val, na.rm = TRUE))
+            maxv <- getM(val)
+            stepv <- (maxv-minv)/100
+            updateSliderInput(
+                session,
+                cellInfoXlim,
+                label = "Xlim range:",
+                min = minv-100*stepv,
+                max = maxv+100*stepv,
+                value = c(minv*0.95, maxv*1.05),
+                step = stepv
+            )
+            val2 <- dataSource()$sc1meta[[
+                dataSource()$sc1conf[
+                    dataSource()$sc1conf$UI == input$GeneExprdrY]$ID]]
+            minv2 <- floor(min(val2, na.rm = TRUE))
+            maxv2 <- getM(val2)
+            stepv2 <- (maxv2-minv2)/100
+            updateSliderInput(
+                session,
+                cellInfoYlim,
+                label = "Ylim range:",
+                min = minv2-100*stepv2,
+                max = maxv2+100*stepv2,
+                value = c(minv2*0.95, maxv2*1.05),
+                step = stepv2
+            )
+        })
+        
         plotX <- reactive({
             scDRcell(
                 inpConf=dataSource()$sc1conf,
@@ -648,7 +700,11 @@ updateCellInfoPlot <-
                 ),
                 editorStatus = ifelse(
                     length(input[[paste0('editorStatus', postfix)]]),
-                    input[[paste0('editorStatus', postfix)]], NA)
+                    input[[paste0('editorStatus', postfix)]], NA),
+                xlim = if(input[[cellInfoXYlimTog]] %% 2 == 1)
+                    input[[cellInfoXlim]] else NULL,
+                ylim = if(input[[cellInfoXYlimTog]] %% 2 == 1)
+                    input[[cellInfoYlim]] else NULL
             )
         })
         updateGeneExprDotPlotUI(
@@ -896,6 +952,43 @@ updateGeneExprPlot <-
                 updateSearchTable(input[[GeneNameLabel]])
             }
         }, ignoreInit=TRUE)
+        
+        geneExprXYlimTog <- paste0('GeneExprxylimTog', postfix)
+        geneExprXlim <- paste0('GeneExprxlim', postfix)
+        geneExprYlim <- paste0('GeneExprylim', postfix)
+        observeEvent(input[[geneExprXYlimTog]],{
+            val <- dataSource()$sc1meta[[
+                dataSource()$sc1conf[
+                    dataSource()$sc1conf$UI == input$GeneExprdrX]$ID]]
+            minv <- floor(min(val, na.rm = TRUE))
+            maxv <- getM(val)
+            stepv <- (maxv-minv)/100
+            updateSliderInput(
+                session,
+                geneExprXlim,
+                label = "Xlim range:",
+                min = minv-100*stepv,
+                max = maxv+100*stepv,
+                value = c(minv*0.95, maxv*1.05),
+                step = stepv
+            )
+            val2 <- dataSource()$sc1meta[[
+                dataSource()$sc1conf[
+                    dataSource()$sc1conf$UI == input$GeneExprdrY]$ID]]
+            minv2 <- floor(min(val2, na.rm = TRUE))
+            maxv2 <- getM(val2)
+            stepv2 <- (maxv2-minv2)/100
+            updateSliderInput(
+                session,
+                geneExprYlim,
+                label = "Ylim range:",
+                min = minv2-100*stepv2,
+                max = maxv2+100*stepv2,
+                value = c(minv2*0.95, maxv2*1.05),
+                step = stepv2
+            )
+        })
+        
         ### plots
         plotX <- reactive({
             scDRgene(
@@ -925,7 +1018,11 @@ updateGeneExprPlot <-
                         0
                     else
                         input[[paste0("GeneExprrg", postfix)]],
-                hideFilterCell = input[[paste0("GeneExprhid", postfix)]]
+                hideFilterCell = input[[paste0("GeneExprhid", postfix)]],
+                xlim = if(input[[geneExprXYlimTog]] %% 2 == 1)
+                    input[[geneExprXlim]] else NULL,
+                ylim = if(input[[geneExprXYlimTog]] %% 2 == 1)
+                    input[[geneExprYlim]] else NULL
             )
         })
         updateGeneExprDotPlotUI(
@@ -975,6 +1072,43 @@ updateSubsetGeneExprPlot <-
                 selected = selected
             )
         })
+        ## set xy lim
+        geneExprXYlimTog <- paste0('GeneExprxylimTog', postfix)
+        geneExprXlim <- paste0('GeneExprxlim', postfix)
+        geneExprYlim <- paste0('GeneExprylim', postfix)
+        observeEvent(input[[geneExprXYlimTog]],{
+            val <- dataSource()$sc1meta[[
+                dataSource()$sc1conf[
+                    dataSource()$sc1conf$UI == input$GeneExprdrX]$ID]]
+            minv <- floor(min(val, na.rm = TRUE))
+            maxv <- getM(val)
+            stepv <- (maxv-minv)/100
+            updateSliderInput(
+                session,
+                geneExprXlim,
+                label = "Xlim range:",
+                min = minv-100*stepv,
+                max = maxv+100*stepv,
+                value = c(minv*0.95, maxv*1.05),
+                step = stepv
+            )
+            val2 <- dataSource()$sc1meta[[
+                dataSource()$sc1conf[
+                    dataSource()$sc1conf$UI == input$GeneExprdrY]$ID]]
+            minv2 <- floor(min(val2, na.rm = TRUE))
+            maxv2 <- getM(val2)
+            stepv2 <- (maxv2-minv2)/100
+            updateSliderInput(
+                session,
+                geneExprYlim,
+                label = "Ylim range:",
+                min = minv2-100*stepv2,
+                max = maxv2+100*stepv2,
+                value = c(minv2*0.95, maxv2*1.05),
+                step = stepv2
+            )
+        })
+        
         ### plots
         plotX <- reactive({
             scDRgene(
@@ -1009,7 +1143,11 @@ updateSubsetGeneExprPlot <-
                 valueFilterKey = input$filterCell,
                 valueFilterCutoff = input$filterCellVal,
                 valueFilterCutoff2 = input$filterCellVal2,
-                hideFilterCell = input[[paste0("GeneExprhid", postfix)]]
+                hideFilterCell = input[[paste0("GeneExprhid", postfix)]],
+                xlim = if(input[[geneExprXYlimTog]] %% 2 == 1)
+                    input[[geneExprXlim]] else NULL,
+                ylim = if(input[[geneExprXYlimTog]] %% 2 == 1)
+                    input[[geneExprYlim]] else NULL
             )
         })
         updateGeneExprDotPlotUI(
@@ -1213,7 +1351,13 @@ relevelCol <- function(inpConf, ui_key, ggData, coln) {
     }
     return(ggCol)
 }
-fixCoord <- function(ggOut, aspectRatio, ratio) {
+fixCoord <- function(ggOut, aspectRatio, ratio, xlim=NULL, ylim=NULL) {
+    if(!is.null(xlim)){
+        ggOut <- ggOut + xlim(xlim)
+    }
+    if(!is.null(ylim)){
+        ggOut <- ggOut + ylim(ylim)
+    }
     if (aspectRatio == "Square") {
         ggOut <- ggOut + coord_fixed(ratio = ratio)
     } else if (aspectRatio == "Fixed") {
@@ -1396,6 +1540,7 @@ namedSubsetCellVals <- function(subsetCellKey, subsetCellVal){
     subsetCellVal[unique(subsetCellKey)]
 }
 
+#' @importFrom data.table .I
 filterCells <- function(
         ggData,
         subsetCellKey,
@@ -1463,17 +1608,100 @@ filterCells <- function(
                 choose <- ok[sample(n, round(n*subsetCellPct))]
                 idx[choose] <- TRUE
             }
-            list(sel = idx)
+            list(sel = idx, orig_order = .I)
         }, by=factor_columns]
         adminMsg(paste0('cell number after/before percentage filter is ',
                         sum(sel$sel),'/', sum(keep), ' (',
                        round(100*sum(sel$sel)/sum(keep), digits = 2), '%)'),
                  type = 'message')
+        orig_order <- sel$orig_order
+        sel <- sel[order(orig_order)]
+        sel$orig_order <- NULL
+        for(coln in colnames(sel)){
+            if(coln!='sel'){
+                stopifnot(identical(ggData[[coln]], sel[[coln]]))
+            }
+        }
         keep <- sel$sel
     }
     return(keep)
 }
 
+getFilteredCellNum <- function(
+        inpConf,
+        inpMeta,
+        dimRedX,
+        dimRedY,
+        cellinfoID,
+        cellinfoName=cellinfoID,
+        subsetCellKey,
+        subsetCellVal,
+        subsetCellPct=100,
+        dataset,
+        geneIdMap,
+        valueFilterKey,
+        valueFilterCutoff,
+        valueFilterCutoff2,
+        ...) {
+    subFilterColname <- 'subValue'
+    subsetCellKey <- subsetCellKey[subsetCellKey!="N/A"]
+    subsetCellVal <- namedSubsetCellVals(subsetCellKey, subsetCellVal)
+    if(cellinfoName!=cellinfoID){
+        if(is.na(cellinfoName)||cellinfoName==""){
+            cellinfoName <- cellinfoID
+        }
+    }
+    # Prepare ggData
+    ggData <- inpMeta[, unique(c(
+        inpConf[inpConf$UI == dimRedX]$ID,
+        inpConf[inpConf$UI == dimRedY]$ID,
+        inpConf[inpConf$UI == cellinfoID]$ID,
+        inpConf[inpConf$UI %in% subsetCellKey]$ID,
+        inpConf[inpConf$UI == cellinfoName]$ID)),
+        with = FALSE]
+    if (ncol(ggData) < 3)
+        return(0)
+    colnames(ggData)[c(1,2)] <- c("X", "Y")
+    dots <- list(...)
+    if('interactive' %in% names(dots)){
+        if(isTRUE(dots$interactive)){
+            ggData$sampleID <- inpMeta$sampleID
+        }
+    }
+    lassoSelected <- rep(TRUE, nrow(ggData))
+    if('selectedCellIDs' %in% names(dots)){
+        if(length(dots$selectedCellIDs)){
+            if(all(dots$selectedCellIDs %in% inpMeta$sampleID)){
+                lassoSelected <- inpMeta$sampleID %in% dots$selectedCellIDs
+            }
+        }
+    }
+    ggData <-
+        cbindFilterValues(
+            ggData,
+            inpConf,
+            inpMeta,
+            subFilterColname,
+            geneIdMap,
+            dataset,
+            valueFilterKey,
+            valueFilterCutoff,
+            valueFilterCutoff2
+        )
+    rat <- getRatio(ggData)
+    keep <- filterCells(
+        ggData,
+        subsetCellKey,
+        subsetCellVal,
+        subFilterColname,
+        valueFilterCutoff,
+        valueFilterCutoff2,
+        inpConf, 
+        subsetCellPct,
+        lassoSelected)
+    return(sum(keep))
+}
+
 updateRankList <- function(input, output, dataSource, uid, pid, input_id){
     observeEvent(input[[uid]], {
         output[[pid]] <- renderUI(rank_list(
